import 'dart:async';
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import 'better_player_ad_controls.dart';
import 'better_player_control.dart';
import 'better_player_ui_press_controls.dart';
import '../tivio_player_colors.dart';
import '../../tivio.dart';
import '../../player/tivio_player_input_source.dart';

class BetterPlayerCustomControls extends StatefulWidget {
  final TivioPlayerInputSource initSource;
  final StreamController<TivioPlayerInputSource> sourceStream;
  final BetterPlayerController controller;
  final StreamController<BetterPlayerEvent> eventPlayerStream;
  final TivioPlayerColors tivioPlayerColors;
  final BetterPlayerUiPressControls betterPlayerUiPressControls;
  final void Function() handleSkipAd;

  BetterPlayerCustomControls({
    required this.initSource,
    required this.sourceStream,
    required this.controller,
    required this.eventPlayerStream,
    required this.tivioPlayerColors,
    required this.betterPlayerUiPressControls,
    required this.handleSkipAd,
  });

  @override
  createState() => _BetterPlayerCustomControlsState();
}

class _BetterPlayerCustomControlsState extends State<BetterPlayerCustomControls> {
  late StreamSubscription sourceStreamSubscription;
  TivioPlayerInputSource? currentSource;

  @override
  void initState() {
    super.initState();

    sourceStreamSubscription = widget.sourceStream.stream.asBroadcastStream().listen((event) {
      setState(() {
        currentSource = event;
      });
    });

    setState(() {
      currentSource = widget.initSource;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: buildPlayerUiBasedOnSourceType(),
    );
  }

  buildPlayerUiBasedOnSourceType() {
    if (currentSource == null) {
      return null;
    }

    if (currentSource!.type == TivioPlayerWrapperInputSourceType.Advertisement) {
      return BetterPlayerAdControls(
        eventPlayerStream: widget.eventPlayerStream,
        onClick: widget.handleSkipAd,
        source: currentSource as TivioPlayerAdSource,
      );
    }

    if (currentSource!.type == TivioPlayerWrapperInputSourceType.Vod) {
      return BetterPlayerControl(
        controller: widget.controller,
        eventPlayerStream: widget.eventPlayerStream,
        tivioPlayerColors: widget.tivioPlayerColors,
        controls: widget.betterPlayerUiPressControls,
        source: currentSource as TivioPlayerVodSource,
      );
    }
  }

  @override
  void dispose() {
    sourceStreamSubscription.cancel();
    super.dispose();
  }
}